# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
test_usage
----------------------------------

Functional tests for `shade` usage method
"""
import datetime

from shade.tests.functional import base


class TestUsage(base.BaseFunctionalTestCase):

    def test_get_usage(self):
        '''Test quotas functionality'''
        usage = self.operator_cloud.get_compute_usage('demo',
                                                      datetime.datetime.now(),
                                                      datetime.datetime.now())
        self.assertIsNotNone(usage)
        self.assertTrue(hasattr(usage, 'total_hours'))
